// bump mapping shader for proj2 - cs594
// based on oZone3D.Net Tutorials
//
// tweaked by Byungil Jeong

varying vec3 lightVec;
varying vec3 eyeVec;
varying vec3 ecCoord;

// textures for bump mapping
uniform sampler2D colorMap;
uniform sampler2D normalMap;

// variables needed to calculate fog
uniform vec3 eyePos;
uniform float waterHeight;

// turn on/off fog effect by application
uniform bool fogOn;

// color of fog
const vec4 waterColor = vec4(0.1, 0.1, 0.3, 0.0);

// compute the distance between the water surface and objects
// or between the water surface and camera
// along the view vector
float computeFog(vec3 ecVertex)
{
	// compute the position to which eye position is projected on the water surface
	// in eye coordinate
	vec4 eyeBase = gl_ModelViewMatrix * vec4(eyePos.x, waterHeight, eyePos.z, 1.0);
	
	// -y dir in eye coordinate
	vec3 baseDir = normalize(eyeBase.xyz);
	
	// the length of the projection of (vertex - eye) onto the -y direction
	float a = dot(ecVertex, baseDir);
	// distance between water surface and eye
	float b = length(eyeBase.xyz);
	
	float fogLen = 0.0;
	
	// eye : upside, vertex : downside
	if (eyePos.y > waterHeight && b < a)
		fogLen = length(ecVertex) * (1-b/a);		
	
	// eye : downside, vertex : downside
	if (eyePos.y < waterHeight && b > a)
		fogLen = length(ecVertex);

	// eye : downside, vertex : upside
	if (eyePos.y < waterHeight && b <= a)
		fogLen = length(ecVertex) * b/a;
			
	return fogLen; 
}

void main (void)
{
	vec3 vVec = normalize(eyeVec);

	// get base color and normal from textures
	vec4 base = texture2D(colorMap, gl_TexCoord[0].st);
	vec3 bump = normalize( texture2D(normalMap, gl_TexCoord[0].st).xyz * 2.0 - 1.0);

	// calculate lighting
	vec4 vAmbient = gl_FrontMaterial.ambient;
	float diffuse = max( dot(lightVec, bump), 0.0 );
	vec4 vDiffuse = gl_FrontMaterial.diffuse * diffuse;	
	float specular = pow(clamp(dot(reflect(-vVec, bump), lightVec), 0.0, 1.0), 
	                 128);
	vec4 vSpecular = gl_FrontMaterial.specular * specular;	
	
	vec4 lightedColor = (vAmbient*base + vDiffuse*base + vSpecular);
	
	// calculate fog effect
	float waterFog;
	if (fogOn)
		waterFog = clamp(computeFog(ecCoord) * 0.018, 0.0, 1.0);
	else
		waterFog = 0.0;
	
	//	get the final color with fog
	gl_FragColor = lightedColor * (1 - waterFog) + waterColor*waterFog;
}
